#include <stdio.h>

#include "util.h"
#include "vpr_types.h"
#define OMEGA  0.20


extern int num_nets, num_blocks;
extern int num_p_inputs, num_p_outputs, num_clbs, num_globals;
extern struct s_net *net;
extern struct s_block *block;
extern char * sens_file;

float lMaxSen = 0.0;

extern char* model;
struct fault_sim *FSIM;
int sNumEdges = 0;
int read_fault_sim()
{
    char lFileName[1024];
    char src[1024];
    char sink[1024];
    float lcrit;
    int lSrcBlkId, lSinkBlkId;
    FILE *fp;
    int i,j,lItem;
    FSIM = (struct fault_sim*) my_malloc(num_blocks*sizeof(struct fault_sim));
    for(i=0;i<num_blocks;i++)
    {
    //    printf(" %d %s \n",i,block[i].name);
        for(j=0;j<NUM_BLK;j++)
        {
            FSIM[i].blkId[j] = -1;
            FSIM[i].crit[j] = 0.0;
        }
        FSIM[i].lCnt = 0;
    }

    fp = fopen(sens_file,"r+");
    if(fp == NULL)
    {
        printf(" FILE %s cannot be opened. Try sens and create a sens file \n",sens_file);
        exit(0);
    }
    while(!feof(fp))
    {
	   fscanf(fp,"%s --> %s = %f\n", src, sink, &lcrit);
	    lSrcBlkId = get_blk_id(src);
	    lSinkBlkId =  get_blk_id(sink);
	    if(FSIM[lSrcBlkId].lCnt > NUM_BLK)
	    {
		    printf(" COUNT OF BLK ID %s gone above limit \n",src);
		    exit(0);
	    }

	    lItem = FSIM[lSrcBlkId].lCnt;
	    //Since its the cluster delay have to see if the data is still there or not.
	    for(j=0;j<lItem;j++)
	    {
		    if(FSIM[lSrcBlkId].blkId[j] == lSinkBlkId) {
			    FSIM[lSrcBlkId].blkId[j] += lcrit; 
			    break; 
		    }
	    }
	    if(j==lItem) //New Item to be added. 
	    {	
		    FSIM[lSrcBlkId].blkId[lItem] = lSinkBlkId;
		    FSIM[lSrcBlkId].crit[lItem] = lcrit;
		    FSIM[lSrcBlkId].lCnt++;
		    sNumEdges++;
	    }
	    if(FSIM[lSrcBlkId].crit[j] > lMaxSen)
		    lMaxSen = FSIM[lSrcBlkId].crit[j];
    }

    if(sNumEdges < 1000) sNumEdges = 1000;

    return 1;
}

int get_blk_id(char *xName)
{
    int i =-1;
    for(i=0;i<num_blocks;i++)
    {
        if(!strcmp(xName, block[i].name))
        {
            return i;
        }
    }
    printf(" BLOCK ID for node %s not found \n",xName);
    return i;
}

float get_crit_gain( int from, int to)
{
    int max ;
    int i;
    max = FSIM[from].lCnt;
    for(i=0;i<max;i++)
    {
        if(FSIM[from].blkId[i] == to)
        {
            //found blk ;
            return (FSIM[from].crit[i]);
        }
    }
    return 0.0;
}


float get_total_crit()
{
    int max ;
    int i;
    int blk;
    int k;
    float lCrit = 0.0;
    int lFound =0;
    for(i=0;i<num_blocks;i++)
    {
        max = FSIM[i].lCnt;
        for(k=0;k<max;k++)
        {
            lCrit += FSIM[i].crit[k];
        }
    }
    printf(" The total sensitivity is %f \n",lCrit);
    return lCrit;
}
float get_sens_cost(int inet)
{
    int source_block, sink_block,ipin;
    source_block = net[inet].blocks[0];
    float lSum=0.0;
    for(ipin=0;ipin<net[inet].num_pins;ipin++)
    {
	sink_block = net[inet].blocks[ipin];
	lSum += get_crit_gain(source_block, sink_block);
    }
    return lSum;
}
